@@ -1,5 +1,5 @@
-/* $Id: driver_evermore.c 6566 2009-11-20 03:51:06Z esr $ */
 /*
+ *
  * This is the gpsd driver for EverMore GPSes operating in binary mode.
  * About the only thing this gives us that NMEA won't is TDOP.
  * But we'll get atomic position reports from it, which is good.
@@ -110,6 +110,8 @@
  * 8d indicates that message 0x8d was sent;
  * c5 is EverMore checksum, other bytes are fixed
  *
+ * This file is Copyright (c) 2010 by the GPSD project
+ * BSD terms apply: see the file COPYING in the distribution root for details. 
  */
 
 #include <sys/types.h>
@@ -134,7 +136,8 @@
 #define EVERMORE_CHANNELS	12
 
 /*@ +charint @*/
-gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size_t len)
+gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
+			  size_t len)
 {
     unsigned char buf2[MAX_PACKET_LENGTH], *cp, *tp;
     size_t i, datalen;
@@ -148,18 +151,20 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
     /* time to unstuff it and discard the header and footer */
     cp = buf + 2;
     tp = buf2;
-    if (*cp == 0x10) cp++;
-    datalen = (size_t)*cp++;
+    if (*cp == 0x10)
+	cp++;
+    datalen = (size_t) * cp++;
 
     gpsd_report(LOG_RAW, "raw EverMore packet type 0x%02x, length %zd: %s\n",
-	*cp, len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
+		*cp, len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
 
     datalen -= 2;
 
     /*@ -usedef @*/
-    for (i = 0; i < (size_t)datalen; i++) {
+    for (i = 0; i < (size_t) datalen; i++) {
 	*tp = *cp++;
-	if (*tp == 0x10) cp++;
+	if (*tp == 0x10)
+	    cp++;
 	tp++;
     }
 
@@ -168,7 +173,7 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 
     /*@ -usedef -compdef @*/
     gpsd_report(LOG_RAW, "EverMore packet type 0x%02x, length %zd: %s\n",
-	type, datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_RAW));
+		type, datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_RAW));
     /*@ +usedef +compdef @*/
 
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
@@ -176,113 +181,123 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 
     session->cycle_end_reliable = true;
 
-    switch (type)
-    {
-    case 0x02:	/* Navigation Data Output */
-	/*@ ignore @*//*@ splint is confused @*/
-	session->gpsdata.fix.time =
-	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+    switch (type) {
+    case 0x02:			/* Navigation Data Output */
+	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
+	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
+	/*@ ignore @*//*@ splint is confused @ */
+	session->newdata.time =
+	    gpstime_to_unix(session->context->gps_week,
+			    session->context->gps_tow) -
+	    session->context->leap_seconds;
 	/*@ end @*/
-	ecef_to_wgs84fix(&session->gpsdata, 
-			 getlesl(buf2, 8)*1.0, getlesl(buf2, 12)*1.0, getlesl(buf2, 16)*1.0,
-			 getlesw(buf2, 20)/10.0, getlesw(buf2, 22)/10.0, getlesw(buf2, 24)/10.0);
+	ecef_to_wgs84fix(&session->newdata, &session->gpsdata.separation,
+			 getlesl(buf2, 8) * 1.0, getlesl(buf2, 12) * 1.0,
+			 getlesl(buf2, 16) * 1.0, getlesw(buf2, 20) / 10.0,
+			 getlesw(buf2, 22) / 10.0, getlesw(buf2, 24) / 10.0);
 	used = (unsigned char)getub(buf2, 26) & 0x0f;
 	//visible = (getub(buf2, 26) & 0xf0) >> 4;
-	version = (uint)getleuw(buf2, 27)/100.0;
+	version = (uint) getleuw(buf2, 27) / 100.0;
 	/* that's all the information in this packet */
 	if (used < 3)
-	    session->gpsdata.fix.mode = MODE_NO_FIX;
+	    session->newdata.mode = MODE_NO_FIX;
 	else if (used == 3)
-	    session->gpsdata.fix.mode = MODE_2D;
+	    session->newdata.mode = MODE_2D;
 	else {
-	    session->gpsdata.fix.mode = MODE_3D;
-	    mask |= ALTITUDE_SET | CLIMB_SET;
+	    session->newdata.mode = MODE_3D;
+	    mask |= ALTITUDE_IS | CLIMB_IS;
 	}
-	mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | MODE_SET;
+	mask |= TIME_IS | LATLON_IS | TRACK_IS | SPEED_IS | MODE_IS;
 	if (session->subtype[0] == '\0') {
-	    (void)snprintf(session->subtype, sizeof(session->subtype), 
-		       "%3.2f", version);
-	    mask |= DEVICEID_SET;
+	    (void)snprintf(session->subtype, sizeof(session->subtype),
+			   "%3.2f", version);
+	    mask |= DEVICEID_IS;
 	}
-	gpsd_report(LOG_DATA, "NDO 0x02: time=%.2f, lat=%.2f lon=%.2f alt=%.2f speed=%.2f track=%.2f climb=%.2f mode=%d subtype='%s' mask=%s\n",
-		    session->gpsdata.fix.time,
-		    session->gpsdata.fix.latitude,
-		    session->gpsdata.fix.longitude,
-		    session->gpsdata.fix.altitude,
-		    session->gpsdata.fix.speed,
-		    session->gpsdata.fix.track,
-		    session->gpsdata.fix.climb,
-		    session->gpsdata.fix.mode,
-		    session->gpsdata.dev.subtype,
-		    gpsd_maskdump(mask));
-	return mask | CLEAR_SET | REPORT_SET;
+	gpsd_report(LOG_DATA,
+		    "NDO 0x02: time=%.2f, lat=%.2f lon=%.2f alt=%.2f speed=%.2f track=%.2f climb=%.2f mode=%d subtype='%s' mask=%s\n",
+		    session->newdata.time, session->newdata.latitude,
+		    session->newdata.longitude, session->newdata.altitude,
+		    session->newdata.speed, session->newdata.track,
+		    session->newdata.climb, session->newdata.mode,
+		    session->gpsdata.dev.subtype, gpsd_maskdump(mask));
+	return mask | CLEAR_IS | REPORT_IS;
 
-    case 0x04:	/* DOP Data Output */
-	/*@ ignore @*//*@ splint is confused @*/
-	session->gpsdata.fix.time =
-	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+    case 0x04:			/* DOP Data Output */
+	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
+	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
+	/*@ ignore @*//*@ splint is confused @ */
+	session->newdata.time =
+	    gpstime_to_unix(session->context->gps_week,
+			    session->context->gps_tow) -
+	    session->context->leap_seconds;
 	/*@ end @*/
 	clear_dop(&session->gpsdata.dop);
-	session->gpsdata.dop.gdop = (double)getub(buf2, 8)*0.1;
-	session->gpsdata.dop.pdop = (double)getub(buf2, 9)*0.1;
-	session->gpsdata.dop.hdop = (double)getub(buf2, 10)*0.1;
-	session->gpsdata.dop.vdop = (double)getub(buf2, 11)*0.1;
-	session->gpsdata.dop.tdop = (double)getub(buf2, 12)*0.1;
+	session->gpsdata.dop.gdop = (double)getub(buf2, 8) * 0.1;
+	session->gpsdata.dop.pdop = (double)getub(buf2, 9) * 0.1;
+	session->gpsdata.dop.hdop = (double)getub(buf2, 10) * 0.1;
+	session->gpsdata.dop.vdop = (double)getub(buf2, 11) * 0.1;
+	session->gpsdata.dop.tdop = (double)getub(buf2, 12) * 0.1;
 	switch (getub(buf2, 13)) {
-	case 0:	/* no position fix */
-	case 1:	/* manual calls this "1D navigation" */
+	case 0:		/* no position fix */
+	case 1:		/* manual calls this "1D navigation" */
 	    session->gpsdata.status = STATUS_NO_FIX;
-	    session->gpsdata.fix.mode = MODE_NO_FIX;
+	    session->newdata.mode = MODE_NO_FIX;
 	    break;
-	case 2:	/* 2D navigation */
+	case 2:		/* 2D navigation */
 	    session->gpsdata.status = STATUS_FIX;
-	    session->gpsdata.fix.mode = MODE_2D;
+	    session->newdata.mode = MODE_2D;
 	    break;
-	case 3:	/* 3D navigation */
+	case 3:		/* 3D navigation */
 	    session->gpsdata.status = STATUS_FIX;
-	    session->gpsdata.fix.mode = MODE_3D;
+	    session->newdata.mode = MODE_3D;
 	    break;
-	case 4:	/* 3D navigation with DGPS */
+	case 4:		/* 3D navigation with DGPS */
 	    session->gpsdata.status = STATUS_DGPS_FIX;
-	    session->gpsdata.fix.mode = MODE_3D;
+	    session->newdata.mode = MODE_3D;
 	    break;
 	}
 	/* that's all the information in this packet */
-	mask = TIME_SET | DOP_SET | MODE_SET | STATUS_SET;
-	gpsd_report(LOG_DATA, "DDO 0x04: gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mode=%d, status=%d mask={TIME| DOP|MODE|STATUS}\n", 
-		    session->gpsdata.dop.gdop,
-		    session->gpsdata.dop.pdop,
-		    session->gpsdata.dop.hdop,
-		    session->gpsdata.dop.vdop,
-		    session->gpsdata.dop.tdop,
-		    session->gpsdata.fix.mode,
+	mask = TIME_IS | DOP_IS | MODE_IS | STATUS_IS;
+	gpsd_report(LOG_DATA,
+		    "DDO 0x04: gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mode=%d, status=%d mask={TIME| DOP|MODE|STATUS}\n",
+		    session->gpsdata.dop.gdop, session->gpsdata.dop.pdop,
+		    session->gpsdata.dop.hdop, session->gpsdata.dop.vdop,
+		    session->gpsdata.dop.tdop, session->newdata.mode,
 		    session->gpsdata.status);
 	return mask;
 
-    case 0x06:	/* Channel Status Output */
-	/*@ ignore @*//*@ splint is confused @*/
+    case 0x06:			/* Channel Status Output */
+	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
+	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
+	/*@ ignore @*//*@ splint is confused @ */
 	session->gpsdata.skyview_time =
-	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	    gpstime_to_unix(session->context->gps_week,
+			    session->context->gps_tow) -
+	    session->context->leap_seconds;
 	/*@ end @*/
 	session->gpsdata.satellites_visible = (int)getub(buf2, 8);
 	session->gpsdata.satellites_used = 0;
 	memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
 	if (session->gpsdata.satellites_visible > 12) {
-		gpsd_report(LOG_WARN, "Warning: EverMore packet has information about %d satellites!\n",
-				session->gpsdata.satellites_visible);
+	    gpsd_report(LOG_WARN,
+			"Warning: EverMore packet has information about %d satellites!\n",
+			session->gpsdata.satellites_visible);
 	}
-	if (session->gpsdata.satellites_visible > EVERMORE_CHANNELS) 
+	if (session->gpsdata.satellites_visible > EVERMORE_CHANNELS)
 	    session->gpsdata.satellites_visible = EVERMORE_CHANNELS;
 	satcnt = 0;
-	for (i = 0; i < (size_t)session->gpsdata.satellites_visible; i++) {
+	for (i = 0; i < (size_t) session->gpsdata.satellites_visible; i++) {
 	    int prn;
 	    // channel = getub(buf2, 7*i+7+2)
-	    prn = (int)getub(buf2, 7*i+7+3);
-	    if (prn == 0) continue;  /* satellite record is not valid */
+	    prn = (int)getub(buf2, 7 * i + 7 + 3);
+	    if (prn == 0)
+		continue;	/* satellite record is not valid */
 	    session->gpsdata.PRN[satcnt] = prn;
-	    session->gpsdata.azimuth[satcnt] = (int)getleuw(buf2, 7*i+7+4);
-	    session->gpsdata.elevation[satcnt] = (int)getub(buf2, 7*i+7+6);
-	    session->gpsdata.ss[satcnt] = (float)getub(buf2, 7*i+7+7);
+	    session->gpsdata.azimuth[satcnt] =
+		(int)getleuw(buf2, 7 * i + 7 + 4);
+	    session->gpsdata.elevation[satcnt] =
+		(int)getub(buf2, 7 * i + 7 + 6);
+	    session->gpsdata.ss[satcnt] = (float)getub(buf2, 7 * i + 7 + 7);
 	    /*
 	     * Status bits at offset 8:
 	     * bit0 = 1 satellite acquired
@@ -293,57 +308,64 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 	     * bit5 = 1 ephemeris data collected
 	     * bit6 = 1 used for position fix
 	     */
-	    if (getub(buf2, 7*i+7+8) & 0x40) {
-		session->gpsdata.used[session->gpsdata.satellites_used++]=prn;
+	    if (getub(buf2, 7 * i + 7 + 8) & 0x40) {
+		session->gpsdata.used[session->gpsdata.satellites_used++] =
+		    prn;
 	    }
 
 	    satcnt++;
 	}
 	session->gpsdata.satellites_visible = (int)satcnt;
 	/* that's all the information in this packet */
-	mask = SATELLITE_SET | USED_SET;
-	gpsd_report(LOG_DATA, "CSO 0x06: time=%.2f used=%d visible=%d mask={TIME|SATELLITE|USED}\n",
-		    session->gpsdata.fix.time,
-		    session->gpsdata.satellites_used,
+	mask = SATELLITE_IS | USED_IS;
+	gpsd_report(LOG_DATA,
+		    "CSO 0x06: time=%.2f used=%d visible=%d mask={TIME|SATELLITE|USED}\n",
+		    session->newdata.time, session->gpsdata.satellites_used,
 		    session->gpsdata.satellites_visible);
 	return mask;
 
-    case 0x08:	/* Measurement Data Output */
+    case 0x08:			/* Measurement Data Output */
 	/* clock offset is a manufacturer diagnostic */
 	/* (int)getleuw(buf2, 8);  clock offset, 29000..29850 ?? */
-	/*@ ignore @*//*@ splint is confused @*/
-	session->gpsdata.fix.time = 
-	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
+	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
+	/*@ ignore @*//*@ splint is confused @ */
+	session->newdata.time =
+	    gpstime_to_unix(session->context->gps_week,
+			    session->context->gps_tow) -
+	    session->context->leap_seconds;
 	/*@ end @*/
 	visible = (unsigned char)getub(buf2, 10);
 	/* FIXME: read full statellite status for each channel */
 	/* we can get pseudo range (m), delta-range (m/s), doppler (Hz) and status for each channel */
 	/* gpsd_report(LOG_PROG, "MDO 0x04: visible=%d\n", visible); */
 	gpsd_report(LOG_DATA, "MDO 0x04: time=%.2f mask={TIME}\n",
-		    session->gpsdata.fix.time);
-	return TIME_SET;
+		    session->newdata.time);
+	return TIME_IS;
 
-    case 0x20:	/* LogConfig Info, could be used as a probe for EverMore GPS */
+    case 0x20:			/* LogConfig Info, could be used as a probe for EverMore GPS */
 	gpsd_report(LOG_IO, "LogConfig EverMore packet, length %zd: %s\n",
-	    datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_IO));
-	return ONLINE_SET;
+		    datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_IO));
+	return ONLINE_IS;
 
-    case 0x22:	/* LogData */
+    case 0x22:			/* LogData */
 	gpsd_report(LOG_IO, "LogData EverMore packet, length %zd: %s\n",
-	    datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_IO));
-	return ONLINE_SET;
+		    datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_IO));
+	return ONLINE_IS;
 
-    case 0x38:	/* ACK */
+    case 0x38:			/* ACK */
 	gpsd_report(LOG_PROG, "EverMore command %02X ACK\n", getub(buf2, 2));
-	return ONLINE_SET;
+	return ONLINE_IS;
 
     default:
 	gpsd_report(LOG_WARN,
-	    "unknown EverMore packet EID 0x%02x, length %zd: %s\n", buf2[0],
-	    datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_IO));
+		    "unknown EverMore packet EID 0x%02x, length %zd: %s\n",
+		    buf2[0], datalen, gpsd_hexdump_wrapper(buf2, datalen,
+							   LOG_IO));
 	return 0;
     }
 }
+
 /*@ -charint @*/
 
 static gps_mask_t evermore_parse_input(struct gps_device_t *session)
@@ -351,7 +373,8 @@ static gps_mask_t evermore_parse_input(struct gps_device_t *session)
     gps_mask_t st;
 
     if (session->packet.type == EVERMORE_PACKET) {
-	st = evermore_parse(session, session->packet.outbuffer, session->packet.outbuflen);
+	st = evermore_parse(session, session->packet.outbuffer,
+			    session->packet.outbuflen);
 	return st;
 #ifdef NMEA_ENABLE
     } else if (session->packet.type == NMEA_PACKET) {
@@ -362,96 +385,70 @@ static gps_mask_t evermore_parse_input(struct gps_device_t *session)
 	return 0;
 }
 
-#ifdef ALLOW_CONTROLSEND
 /*@ +charint -usedef -compdef @*/
-static ssize_t evermore_control_send(struct gps_device_t *session, char *buf, size_t len)
+static ssize_t evermore_control_send(struct gps_device_t *session, char *buf,
+				     size_t len)
 {
-   unsigned int       crc;
-   size_t    i;
-   char *cp;
+    unsigned int crc;
+    size_t i;
+    char *cp;
 
-   /*@ +charint +ignoresigns @*/
-   /* prepare a DLE-stuffed copy of the message */
-   cp = session->msgbuf;
-   *cp++ = 0x10;  /* message starts with DLE STX */
-   *cp++ = 0x02;
+    /*@ +charint +ignoresigns @*/
+    /* prepare a DLE-stuffed copy of the message */
+    cp = session->msgbuf;
+    *cp++ = 0x10;		/* message starts with DLE STX */
+    *cp++ = 0x02;
 
-   session->msgbuflen = (size_t)(len + 2);   /* len < 254 !! */
-   *cp++ = (char)session->msgbuflen;   /* message length */
-   if (session->msgbuflen == 0x10) *cp++ = 0x10;
-   
-   /* payload */
-   crc = 0;
-   for (i = 0; i < len; i++) {
-       *cp++ = buf[i];
-       if (buf[i] == 0x10) 
-	   *cp++ = 0x10;
-       crc += buf[i];
-   }
+    session->msgbuflen = (size_t) (len + 2);	/* len < 254 !! */
+    *cp++ = (char)session->msgbuflen;	/* message length */
+    if (session->msgbuflen == 0x10)
+	*cp++ = 0x10;
 
-   crc &= 0xff;
-
-   /* enter CRC after payload */
-   *cp++ = crc;  
-   if (crc == 0x10)
-       *cp++ = 0x10;
-
-   *cp++ = 0x10;   /* message ends with DLE ETX */
-   *cp++ = 0x03;
-
-   session->msgbuflen = (size_t)(cp - session->msgbuf);
-   /*@ -charint -ignoresigns @*/
-
-   return gpsd_write(session, session->msgbuf, session->msgbuflen);
-}
-/*@ -charint +usedef +compdef @*/
-#endif /* ALLOW_CONTROLSEND */
-
-#ifdef ALLOW_RECONFIGURE
-static bool evermore_speed(struct gps_device_t *session, 
-			   speed_t speed, char parity, int stopbits)
-{
-    /*@ -type @*/
-    gpsd_report(LOG_PROG, "evermore_speed(%u%c%d)\n", speed, parity, stopbits);
-    /* parity and stopbit switching aren't available on this chip */
-    if (parity!=session->gpsdata.dev.parity || stopbits!=(int)session->gpsdata.dev.parity) {
-	return false;
-    } else {
-	unsigned char tmp8;
-	char msg[] = {
-	    0x89,          /*  0: msg ID, Serial Port Configuration */
-	    0x01,          /*  1: bit 0 cfg for main serial, bit 1 cfg for DGPS port */
-	    0x00,          /*  2: baud rate for main serial; 4800(0), 9600(1), 19200(2), 38400(3) */
-	    0x00,          /*  3: baud rate for DGPS serial port; 4800(0), 9600(1), etc */
-	};
-	switch (speed) {
-	case 4800:  tmp8 = 0; break;
-	case 9600:  tmp8 = 1; break;
-	case 19200: tmp8 = 2; break;
-	case 38400: tmp8 = 3; break;
-	default: return false;
-	}
-	msg[2] = tmp8;
-	return (evermore_control_send(session, msg, sizeof(msg)) != -1);
+    /* payload */
+    crc = 0;
+    for (i = 0; i < len; i++) {
+	*cp++ = buf[i];
+	if (buf[i] == 0x10)
+	    *cp++ = 0x10;
+	crc += buf[i];
     }
-    /*@ +type @*/
+
+    crc &= 0xff;
+
+    /* enter CRC after payload */
+    *cp++ = crc;
+    if (crc == 0x10)
+	*cp++ = 0x10;
+
+    *cp++ = 0x10;		/* message ends with DLE ETX */
+    *cp++ = 0x03;
+
+    session->msgbuflen = (size_t) (cp - session->msgbuf);
+    /*@ -charint -ignoresigns @*/
+
+    return gpsd_write(session, session->msgbuf, session->msgbuflen);
 }
 
+/*@ -charint +usedef +compdef @*/
+
 static bool evermore_protocol(struct gps_device_t *session, int protocol)
 {
     /*@ +charint */
     char tmp8;
     char evrm_protocol_config[] = {
-	    0x84,    /* 0: msg ID, Protocol Configuration */
-	    0x00,    /* 1: mode; EverMore binary(0), NMEA(1) */
-	    0x00,    /* 2: reserved */
-	    0x00,    /* 3: reserved */
+	(char)0x84,		/* 0: msg ID, Protocol Configuration */
+	(char)0x00,		/* 1: mode; EverMore binary(0), NMEA(1) */
+	(char)0x00,		/* 2: reserved */
+	(char)0x00,		/* 3: reserved */
     };
     /*@ -charint */
     gpsd_report(LOG_PROG, "evermore_protocol(%d)\n", protocol);
-    /*@i1@*/tmp8 = (protocol != 0) ? 1 : 0;   /* NMEA : binary */
+    /*@i1@*/ tmp8 = (protocol != 0) ? 1 : 0;
+    /* NMEA : binary */
     evrm_protocol_config[1] = tmp8;
-    return (evermore_control_send(session, evrm_protocol_config, sizeof(evrm_protocol_config)) != -1);
+    return (evermore_control_send
+	    (session, evrm_protocol_config,
+	     sizeof(evrm_protocol_config)) != -1);
 }
 
 static bool evermore_nmea_config(struct gps_device_t *session, int mode)
@@ -459,43 +456,47 @@ static bool evermore_nmea_config(struct gps_device_t *session, int mode)
 /* mode = 1 : gpsd best */
 /* mode = 2 : EverMore search, activate PEMT101 message */
 {
-    char tmp8;
+    unsigned char tmp8;
     /*@ +charint */
-    char evrm_nmeaout_config[] = {
-	    0x8e,	/*  0: msg ID, NMEA Message Control */
-	    0xff,       /*  1: NMEA sentence bitmask, GGA(0), GLL(1), GSA(2), GSV(3), ... */
-	    0x01,       /*  2: nmea checksum no(0), yes(1) */
-	    1,          /*  3: GPGGA, interval 0-255s */
-	    0,          /*  4: GPGLL, interval 0-255s */
-	    1,          /*  5: GPGSA, interval 0-255s */
-	    1,          /*  6: GPGSV, interval 0-255s */
-	    1,          /*  7: GPRMC, interval 0-255s */
-	    0,          /*  8: GPVTG, interval 0-255s */
-	    0,          /*  9: PEMT,101, interval 0-255s */
-	    0, 0, 0, 0, 0, 0, /* 10-15: reserved */
+    unsigned char evrm_nmeaout_config[] = {
+	0x8e,			/*  0: msg ID, NMEA Message Control */
+	0xff,			/*  1: NMEA sentence bitmask, GGA(0), GLL(1), GSA(2), GSV(3), ... */
+	0x01,			/*  2: nmea checksum no(0), yes(1) */
+	1,			/*  3: GPGGA, interval 0-255s */
+	0,			/*  4: GPGLL, interval 0-255s */
+	1,			/*  5: GPGSA, interval 0-255s */
+	1,			/*  6: GPGSV, interval 0-255s */
+	1,			/*  7: GPRMC, interval 0-255s */
+	0,			/*  8: GPVTG, interval 0-255s */
+	0,			/*  9: PEMT,101, interval 0-255s */
+	0, 0, 0, 0, 0, 0,	/* 10-15: reserved */
     };
     /*@ -charint */
     gpsd_report(LOG_PROG, "evermore_nmea_config(%d)\n", mode);
-    /*@i1@*/tmp8 = (mode == 1) ? 5 : 1;   /* NMEA GPGSV, gpsd  */
-    evrm_nmeaout_config[6] = tmp8;           /* GPGSV, 1s or 5s */
-    /*@i1@*/tmp8 = (mode == 2) ? 1 : 0;   /* NMEA PEMT101 */
-    evrm_nmeaout_config[9] = tmp8;           /* PEMT101, 1s or 0s */
-    return (evermore_control_send(session, evrm_nmeaout_config, sizeof(evrm_nmeaout_config)) != -1);
+    /*@i1@*/ tmp8 = (mode == 1) ? 5 : 1;
+    /* NMEA GPGSV, gpsd  */
+    evrm_nmeaout_config[6] = tmp8;	/* GPGSV, 1s or 5s */
+    /*@i1@*/ tmp8 = (mode == 2) ? 1 : 0;
+    /* NMEA PEMT101 */
+    evrm_nmeaout_config[9] = tmp8;	/* PEMT101, 1s or 0s */
+    return (evermore_control_send(session, (char *)evrm_nmeaout_config,
+				  sizeof(evrm_nmeaout_config)) != -1);
 }
 
 static void evermore_mode(struct gps_device_t *session, int mode)
 {
-    gpsd_report(LOG_PROG, "evermore_mode(%d), %d\n", mode, session->back_to_nmea ? 1 : 0);
+    gpsd_report(LOG_PROG, "evermore_mode(%d), %d\n", mode,
+		session->back_to_nmea ? 1 : 0);
     if (mode == MODE_NMEA) {
-        /* NMEA */
-        (void) evermore_protocol(session, 1);
-        session->gpsdata.dev.driver_mode = MODE_NMEA;
-        (void) evermore_nmea_config(session, 1); /* configure NMEA messages for gpsd */
+	/* NMEA */
+	(void)evermore_protocol(session, 1);
+	session->gpsdata.dev.driver_mode = MODE_NMEA;
+	(void)evermore_nmea_config(session, 1);	/* configure NMEA messages for gpsd */
     } else {
-        /* binary */
-        (void) evermore_protocol(session, 0);
-        session->back_to_nmea = false;
-        session->gpsdata.dev.driver_mode = MODE_BINARY;
+	/* binary */
+	(void)evermore_protocol(session, 0);
+	session->back_to_nmea = false;
+	session->gpsdata.dev.driver_mode = MODE_BINARY;
     }
 }
 
@@ -506,34 +507,76 @@ static void evermore_event_hook(struct gps_device_t *session, event_t event)
      * Experiment to see if the holds its settings through a close.
      */
     if (event == event_identified || event == event_reactivate) {
-        if (session->packet.type == NMEA_PACKET) {
-	    (void) evermore_nmea_config(session, 1); /* configure NMEA messages for gpsd (GPGSV every 5s) */
-        } 
-        (void) evermore_mode(session, 1); /* switch GPS to binary mode */
-        session->back_to_nmea = true;
+	if (session->packet.type == NMEA_PACKET) {
+	    (void)evermore_nmea_config(session, 1);	/* configure NMEA messages for gpsd (GPGSV every 5s) */
+	}
+	(void)evermore_mode(session, 1);	/* switch GPS to binary mode */
+	session->back_to_nmea = true;
     } else if (event == event_deactivate) {
-	(void) evermore_nmea_config(session, 0); /* configure NMEA messages to default */
+	(void)evermore_nmea_config(session, 0);	/* configure NMEA messages to default */
     }
 }
 
+#ifdef ALLOW_RECONFIGURE
+static bool evermore_speed(struct gps_device_t *session,
+			   speed_t speed, char parity, int stopbits)
+{
+    /*@ -type @*/
+    gpsd_report(LOG_PROG, "evermore_speed(%u%c%d)\n", speed, parity,
+		stopbits);
+    /* parity and stopbit switching aren't available on this chip */
+    if (parity != session->gpsdata.dev.parity
+	|| stopbits != (int)session->gpsdata.dev.parity) {
+	return false;
+    } else {
+	unsigned char tmp8;
+	unsigned char msg[] = {
+	    0x89,		/*  0: msg ID, Serial Port Configuration */
+	    0x01,		/*  1: bit 0 cfg for main serial, bit 1 cfg for DGPS port */
+	    0x00,		/*  2: baud rate for main serial; 4800(0), 9600(1), 19200(2), 38400(3) */
+	    0x00,		/*  3: baud rate for DGPS serial port; 4800(0), 9600(1), etc */
+	};
+	switch (speed) {
+	case 4800:
+	    tmp8 = 0;
+	    break;
+	case 9600:
+	    tmp8 = 1;
+	    break;
+	case 19200:
+	    tmp8 = 2;
+	    break;
+	case 38400:
+	    tmp8 = 3;
+	    break;
+	default:
+	    return false;
+	}
+	msg[2] = tmp8;
+	return (evermore_control_send(session, (char *)msg, sizeof(msg)) !=
+		-1);
+    }
+    /*@ +type @*/
+}
 
 static bool evermore_rate_switcher(struct gps_device_t *session, double rate)
 /* change the sample rate of the GPS */
 {
     /*@ +charint @*/
-    char evrm_rate_config[] = {
-	    0x84,    /* 1: msg ID, Operating Mode Configuration */
-	    0x02,    /* 2: normal mode with 1PPS */
-	    0x00,    /* 3: navigation update rate */
-	    0x00,    /* 4: RF/GPSBBP On Time */
+    unsigned char evrm_rate_config[] = {
+	0x84,			/* 1: msg ID, Operating Mode Configuration */
+	0x02,			/* 2: normal mode with 1PPS */
+	0x00,			/* 3: navigation update rate */
+	0x00,			/* 4: RF/GPSBBP On Time */
     };
 
     if (rate < 1 || rate > 10) {
 	gpsd_report(LOG_ERROR, "valid rate range is 1-10.\n");
 	return false;
     } else {
-	evrm_rate_config[2] = (char)trunc(rate);
-	return (evermore_control_send(session, evrm_rate_config, sizeof(evrm_rate_config)) != -1);
+	evrm_rate_config[2] = (unsigned char)trunc(rate);
+	return (evermore_control_send(session, (char *)evrm_rate_config,
+				      sizeof(evrm_rate_config)) != -1);
     }
     /*@ -charint @*/
 }
@@ -541,6 +584,7 @@ static bool evermore_rate_switcher(struct gps_device_t *session, double rate)
 
 
 /* this is everything we export */
+/* *INDENT-OFF* */
 const struct gps_type_t evermore_binary =
 {
     .type_name      = "EverMore binary",	/* full name of type */
@@ -561,5 +605,9 @@ const struct gps_type_t evermore_binary =
 #ifdef ALLOW_CONTROLSEND
     .control_send   = evermore_control_send,	/* how to send a control string */
 #endif /* ALLOW_CONTROLSEND */
+#ifdef NTPSHM_ENABLE
+    .ntp_offset     = NULL,		/* no method for NTP fudge factor */
+#endif /* NTPSHM_ ENABLE */
 };
+/* *INDENT-ON* */
 #endif /* defined(EVERMORE_ENABLE) && defined(BINARY_ENABLE) */
