@@ -1,4 +1,7 @@
-/* $Id: gpxlogger.c 6920 2010-01-12 19:22:47Z esr $ */
+/*
+ * This file is Copyright (c) 2010 by the GPSD project
+ * BSD terms apply: see the file COPYING in the distribution root for details.
+ */
 #include <stdlib.h>
 #include "gpsd_config.h"
 #include <sys/types.h>
@@ -17,6 +20,11 @@
 #endif /* S_SPLINT_S */
 #include "gps.h"
 #include "gpsdclient.h"
+#include "revision.h"
+
+#ifdef S_SPLINT_S
+extern struct tm *gmtime_r(const time_t *, /*@out@*/ struct tm *tp);
+#endif /* S_SPLINT_S */
 
 /**************************************************************************
  *
@@ -31,18 +39,22 @@ static char *progname;
 static time_t int_time, old_int_time;
 static bool intrack = false;
 static bool first = true;
-static time_t timeout = 5; /* seconds */
+static time_t timeout = 5;	/* seconds */
 #ifdef CLIENTDEBUG_ENABLE
 static int debug;
 #endif /* CLIENTDEBUG_ENABLE */
 
-static void print_gpx_header(void) 
+static void print_gpx_header(void)
 {
     (void)printf("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
     (void)printf("<gpx version=\"1.1\" creator=\"navsys logger\"\n");
-    (void)printf("        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
+    (void)
+	printf
+	("        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
     (void)printf("        xmlns=\"http://www.topografix.com/GPX/1.1\"\n");
-    (void)printf("        xsi:schemaLocation=\"http://www.topografix.com/GPS/1/1\n");
+    (void)
+	printf
+	("        xsi:schemaLocation=\"http://www.topografix.com/GPS/1/1\n");
     (void)printf("        http://www.topografix.com/GPX/1/1/gpx.xsd\">\n");
     (void)printf(" <metadata>\n");
     (void)printf("  <name>NavSys GPS logger dump</name>\n");
@@ -52,14 +64,14 @@ static void print_gpx_header(void)
     (void)fflush(stdout);
 }
 
-static void print_gpx_trk_end (void) 
+static void print_gpx_trk_end(void)
 {
     (void)printf("  </trkseg>\n");
     (void)printf(" </trk>\n");
     (void)fflush(stdout);
 }
 
-static void print_gpx_footer (void) 
+static void print_gpx_footer(void)
 {
     if (intrack)
 	print_gpx_trk_end();
@@ -67,7 +79,7 @@ static void print_gpx_footer (void)
     (void)fclose(stdout);
 }
 
-static void print_gpx_trk_start (void)
+static void print_gpx_trk_start(void)
 {
     (void)printf(" <trk>\n");
     (void)printf("  <trkseg>\n");
@@ -76,17 +88,16 @@ static void print_gpx_trk_start (void)
 
 static void print_fix(struct gps_fix_t *fix, struct tm *time)
 {
-    (void)printf("   <trkpt lat=\"%f\" lon=\"%f\">\n", 
+    (void)printf("   <trkpt lat=\"%f\" lon=\"%f\">\n",
 		 fix->latitude, fix->longitude);
-    (void)printf("    <ele>%f</ele>\n",
-		 fix->altitude);
+    (void)printf("    <ele>%f</ele>\n", fix->altitude);
     (void)printf("    <time>%04d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
-		 time->tm_year+1900, time->tm_mon+1, time->tm_mday,
+		 time->tm_year + 1900, time->tm_mon + 1, time->tm_mday,
 		 time->tm_hour, time->tm_min, time->tm_sec);
-    if (fix->mode==MODE_NO_FIX)
-	(void)fprintf (stdout, "    <fix>none</fix>\n");
+    if (fix->mode == MODE_NO_FIX)
+	(void)fprintf(stdout, "    <fix>none</fix>\n");
     else
-	(void)fprintf (stdout, "    <fix>%dd</fix>\n", fix->mode);
+	(void)fprintf(stdout, "    <fix>%dd</fix>\n", fix->mode);
 #if 0
     /*
      * Can't print this more detailed report because in D-Bus mode
@@ -95,19 +106,19 @@ static void print_fix(struct gps_fix_t *fix, struct tm *time)
      */
     if ((gpsdata->status >= 2) && (gpsdata->fix.mode >= MODE_3D)) {
 	/* dgps or pps */
-	if (gpsdata->fix.mode == 4) { /* military pps */
+	if (gpsdata->fix.mode == 4) {	/* military pps */
 	    (void)printf("        <fix>pps</fix>\n");
-	} else { /* civilian dgps or sbas */
+	} else {		/* civilian dgps or sbas */
 	    (void)printf("        <fix>dgps</fix>\n");
 	}
-    } else { /* no dgps or pps */
+    } else {			/* no dgps or pps */
 	if (gpsdata->fix.mode == MODE_3D) {
 	    (void)printf("        <fix>3d</fix>\n");
 	} else if (gpsdata->fix.mode == MODE_2D) {
 	    (void)printf("        <fix>2d</fix>\n");
 	} else if (gpsdata->fix.mode == MODE_NOFIX) {
 	    (void)printf("        <fix>none</fix>\n");
-	} /* don't print anything if no fix indicator */
+	}			/* don't print anything if no fix indicator */
     }
 
     /* print # satellites used in fix, if reasonable to do so */
@@ -118,14 +129,14 @@ static void print_fix(struct gps_fix_t *fix, struct tm *time)
 #endif
 
     (void)printf("   </trkpt>\n");
-    (void)fflush (stdout);
+    (void)fflush(stdout);
 }
 
 static void conditionally_log_fix(struct gps_fix_t *gpsfix)
 {
-    int_time = floor(gpsfix->time);
+    int_time = (time_t) floor(gpsfix->time);
     if ((int_time != old_int_time) && gpsfix->mode >= MODE_2D) {
-	struct tm 	time;
+	struct tm time;
 	/* 
 	 * Make new track if the jump in time is above
 	 * timeout.  Handle jumps both forward and
@@ -133,10 +144,12 @@ static void conditionally_log_fix(struct gps_fix_t *gpsfix)
 	 * backward when gpsd is submitting junk on the
 	 * dbus.
 	 */
+	/*@-type@*/
 	if (fabs(int_time - old_int_time) > timeout && !first) {
 	    print_gpx_trk_end();
 	    intrack = false;
 	}
+	/*@+type@*/
 
 	if (!intrack) {
 	    print_gpx_trk_start();
@@ -144,19 +157,19 @@ static void conditionally_log_fix(struct gps_fix_t *gpsfix)
 	    if (first)
 		first = false;
 	}
-		
+
 	old_int_time = int_time;
-	gmtime_r(&(int_time), &time);
+	(void)gmtime_r(&(int_time), &time);
 	print_fix(gpsfix, &time);
     }
 }
 
-static void quit_handler (int signum) 
+static void quit_handler(int signum)
 {
     /* don't clutter the logs on Ctrl-C */
     if (signum != SIGINT)
 	syslog(LOG_INFO, "exiting, signal %d received", signum);
-    print_gpx_footer ();
+    print_gpx_footer();
     exit(0);
 }
 
@@ -182,36 +195,35 @@ static struct gps_fix_t gpsfix;
  */
 #define EMIX(x, y)	(((x) > (y)) ? (x) : (y))
 
-DBusConnection* connection;
+DBusConnection *connection;
 
 static char gpsd_devname[BUFSIZ];
 
-static DBusHandlerResult handle_gps_fix (DBusMessage* message)
+static DBusHandlerResult handle_gps_fix(DBusMessage * message)
 {
-    DBusError	error;
+    DBusError error;
     /* this packet format was designed before we split eph */
     double eph = EMIX(gpsfix.epx, gpsfix.epy);
 
-    dbus_error_init (&error);
+    dbus_error_init(&error);
 
-    dbus_message_get_args (message,
-			   &error,
-			   DBUS_TYPE_DOUBLE, &gpsfix.time,
-			   DBUS_TYPE_INT32,  &gpsfix.mode,
-			   DBUS_TYPE_DOUBLE, &gpsfix.ept,
-			   DBUS_TYPE_DOUBLE, &gpsfix.latitude,
-			   DBUS_TYPE_DOUBLE, &gpsfix.longitude,
-			   DBUS_TYPE_DOUBLE, &eph,
-			   DBUS_TYPE_DOUBLE, &gpsfix.altitude,
-			   DBUS_TYPE_DOUBLE, &gpsfix.epv,
-			   DBUS_TYPE_DOUBLE, &gpsfix.track,
-			   DBUS_TYPE_DOUBLE, &gpsfix.epd,
-			   DBUS_TYPE_DOUBLE, &gpsfix.speed,
-			   DBUS_TYPE_DOUBLE, &gpsfix.eps,
-			   DBUS_TYPE_DOUBLE, &gpsfix.climb,
-			   DBUS_TYPE_DOUBLE, &gpsfix.epc,
-			   DBUS_TYPE_STRING, &gpsd_devname,
-			   DBUS_TYPE_INVALID);
+    dbus_message_get_args(message,
+			  &error,
+			  DBUS_TYPE_DOUBLE, &gpsfix.time,
+			  DBUS_TYPE_INT32, &gpsfix.mode,
+			  DBUS_TYPE_DOUBLE, &gpsfix.ept,
+			  DBUS_TYPE_DOUBLE, &gpsfix.latitude,
+			  DBUS_TYPE_DOUBLE, &gpsfix.longitude,
+			  DBUS_TYPE_DOUBLE, &eph,
+			  DBUS_TYPE_DOUBLE, &gpsfix.altitude,
+			  DBUS_TYPE_DOUBLE, &gpsfix.epv,
+			  DBUS_TYPE_DOUBLE, &gpsfix.track,
+			  DBUS_TYPE_DOUBLE, &gpsfix.epd,
+			  DBUS_TYPE_DOUBLE, &gpsfix.speed,
+			  DBUS_TYPE_DOUBLE, &gpsfix.eps,
+			  DBUS_TYPE_DOUBLE, &gpsfix.climb,
+			  DBUS_TYPE_DOUBLE, &gpsfix.epc,
+			  DBUS_TYPE_STRING, &gpsd_devname, DBUS_TYPE_INVALID);
 
     conditionally_log_fix(&gpsfix);
     return DBUS_HANDLER_RESULT_HANDLED;
@@ -221,49 +233,52 @@ static DBusHandlerResult handle_gps_fix (DBusMessage* message)
  * Message dispatching function
  *
  */
-static DBusHandlerResult signal_handler (
-		DBusConnection* connection, DBusMessage* message)
+static DBusHandlerResult signal_handler(DBusConnection * connection,
+					DBusMessage * message)
 {
     /* dummy, need to use the variable for some reason */
     connection = NULL;
-	
-    if (dbus_message_is_signal (message, "org.gpsd", "fix")) 
-	return handle_gps_fix (message);
+
+    if (dbus_message_is_signal(message, "org.gpsd", "fix"))
+	return handle_gps_fix(message);
     /*
      * ignore all other messages
      */
-	
+
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
 static int dbus_mainloop(void)
 {
-    GMainLoop* mainloop;
+    GMainLoop *mainloop;
     DBusError error;
 
-    mainloop = g_main_loop_new (NULL, FALSE);
+    mainloop = g_main_loop_new(NULL, FALSE);
 
-    dbus_error_init (&error);
-    connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-    if (dbus_error_is_set (&error)) {
-	syslog (LOG_CRIT, "%s: %s", error.name, error.message);
+    dbus_error_init(&error);
+    connection = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
+    if (dbus_error_is_set(&error)) {
+	syslog(LOG_CRIT, "%s: %s", error.name, error.message);
 	return 3;
     }
-	
-    dbus_bus_add_match (connection, "type='signal'", &error);
-    if (dbus_error_is_set (&error)) {
-	syslog (LOG_CRIT, "unable to add match for signals %s: %s", error.name, error.message);
+
+    dbus_bus_add_match(connection, "type='signal'", &error);
+    if (dbus_error_is_set(&error)) {
+	syslog(LOG_CRIT, "unable to add match for signals %s: %s", error.name,
+	       error.message);
 	return 4;
     }
 
-    if (!dbus_connection_add_filter (connection, (DBusHandleMessageFunction)signal_handler, NULL, NULL)) {
-	syslog (LOG_CRIT, "unable to register filter with the connection");
+    if (!dbus_connection_add_filter
+	(connection, (DBusHandleMessageFunction) signal_handler, NULL,
+	 NULL)) {
+	syslog(LOG_CRIT, "unable to register filter with the connection");
 	return 5;
     }
-	
-    dbus_connection_setup_with_g_main (connection, NULL);
 
-    g_main_loop_run (mainloop);
+    dbus_connection_setup_with_g_main(connection, NULL);
+
+    g_main_loop_run(mainloop);
     return 0;
 }
 
@@ -275,18 +290,20 @@ static int dbus_mainloop(void)
  *
  **************************************************************************/
 
-struct fixsource_t source;
+static struct fixsource_t source;
 
 static void process(struct gps_data_t *gpsdata,
-	     char *buf UNUSED, size_t len UNUSED)
+		    char *buf UNUSED, size_t len UNUSED)
 {
     /* this is where we implement source-device filtering */
-    if (gpsdata->dev.path[0] && source.device!=NULL && strcmp(source.device, gpsdata->dev.path) != 0)
+    if (gpsdata->dev.path[0] != '\0' && source.device != NULL
+	&& strcmp(source.device, gpsdata->dev.path) != 0)
 	return;
 
     conditionally_log_fix(&gpsdata->fix);
 }
 
+/*@-mustfreefresh@*/
 static int socket_mainloop(void)
 {
     fd_set fds;
@@ -301,9 +318,9 @@ static int socket_mainloop(void)
     }
 
     gps_set_raw_hook(gpsdata, process);
-    gps_stream(gpsdata, WATCH_ENABLE|WATCH_NEWSTYLE, NULL);
+    (void)gps_stream(gpsdata, WATCH_ENABLE, NULL);
 
-    for(;;) {
+    for (;;) {
 	int data;
 	struct timeval tv;
 
@@ -315,47 +332,47 @@ static int socket_mainloop(void)
 	data = select(gpsdata->gps_fd + 1, &fds, NULL, NULL, &tv);
 
 	if (data == -1) {
-	    (void)fprintf(stderr,"%s\n", strerror(errno));
+	    (void)fprintf(stderr, "%s\n", strerror(errno));
 	    break;
-	}
-	else if (data)
-	    gps_poll(gpsdata);
+	} else if (data)
+	    (void)gps_poll(gpsdata);
     }
     return 0;
 }
 
+/*@+mustfreefresh@*/
+
 /**************************************************************************
  *
  * Main sequence
  *
  **************************************************************************/
 
-static void usage(void) 
+static void usage(void)
 {
     fprintf(stderr,
 	    "Usage: %s [-V] [-h] [-i timeout] [-j casoc] [server[:port:[device]]]\n",
 	    progname);
     fprintf(stderr,
-	    "\tdefaults to '%s -i 5 -j 0 localhost:2947'\n",
-	    progname);
+	    "\tdefaults to '%s -i 5 -j 0 localhost:2947'\n", progname);
     exit(1);
 }
 
-int main (int argc, char** argv) 
+int main(int argc, char **argv)
 {
     int ch;
 
     progname = argv[0];
     while ((ch = getopt(argc, argv, "D:hi:V")) != -1) {
 	switch (ch) {
+#ifdef CLIENTDEBUG_ENABLE
 	case 'D':
 	    debug = atoi(optarg);
-#ifdef CLIENTDEBUG_ENABLE
 	    gps_enable_debug(debug, stdout);
-#endif /* CLIENTDEBUG_ENABLE */
 	    break;
+#endif /* CLIENTDEBUG_ENABLE */
 	case 'i':		/* set polling interfal */
-	    timeout = (unsigned int)atoi(optarg);
+	    timeout = (time_t) atoi(optarg);
 	    if (timeout < 1)
 		timeout = 1;
 	    if (timeout >= 3600)
@@ -363,10 +380,10 @@ int main (int argc, char** argv)
 			"WARNING: track timeout is an hour or more!\n");
 	    break;
 	case 'V':
-	    (void)fprintf(stderr, "SVN ID: $Id: gpxlogger.c 6920 2010-01-12 19:22:47Z esr $ \n");
+	    (void)fprintf(stderr, "gpxlogger revision " REVISION "\n");
 	    exit(0);
 	default:
- 	    usage();
+	    usage();
 	    /* NOTREACHED */
 	}
     }
@@ -384,14 +401,14 @@ int main (int argc, char** argv)
     gps_clear_fix(&gpsfix);
 
     /* catch all interesting signals */
-    signal (SIGTERM, quit_handler);
-    signal (SIGQUIT, quit_handler);
-    signal (SIGINT, quit_handler);
+    (void)signal(SIGTERM, quit_handler);
+    (void)signal(SIGQUIT, quit_handler);
+    (void)signal(SIGINT, quit_handler);
 
     //openlog ("gpxlogger", LOG_PID | LOG_NDELAY , LOG_DAEMON);
     //syslog (LOG_INFO, "---------- STARTED ----------");
-	
-    print_gpx_header ();
+
+    print_gpx_header();
 
 #ifdef DBUS_ENABLE
     /* To force socket use in the default way just give a 'localhost' arg */
